import React,{ Component } from 'react';

class PhoneInfo extends Component{
    static defaultProps={
        info:{
            name:'이름',
            phone: '010-0000-0000',
            id: 0
        },
        
    }
    state={       
        editing: false, //flag. true이면 수정모드. false이면 기본모드
        name:"",
        phone:""
    }

    //삭제버튼 클릭시 호출
    handleRemove=()=>{
        //구조분해할당
        const {info, onRemove}=this.props;
        // const info=this.props.info;
        // const onRemove=this.props.onRemove;

        //App컴포넌트의 handleRemove가 호출됨
        onRemove(info.id);
    }
    //editing을 toggle하는 함수
    handleToggleEdit=()=>{
        this.setState({ editing: !this.state.editing });
    }
    //input태그의 값이 변경되면 state를 변경
    handleChange=(e)=>{
        const { name, value }=e.target; //구조분해할당
        this.setState({
            [name]:value
        });
    }
    //컴포넌트가 변경될 때(state값이 변경되면) 호출
    componentDidUpdate(prevProps,prevState){
        const { info, onUpdate}=this.props;//구조분해할당
        //editing이 false->true로 바뀔때. 일반모드->수정모드. 수정버튼을 누를 때
        if(!prevState.editing && this.state.editing){
            this.setState({name:info.name,phone:info.phone});
        }
        //eiditing이 true->false로 바뀔때. 수정모드=>일반모드. 적용버튼을 누를 때
        if(prevState.editing && !this.state.editing){
            //App컴포넌트의 handleUpdate호출
            onUpdate(info.id,{
                name:this.state.name,
                phone:this.state.phone
            });
        }
    }
    
    Q//컴포넌트가 수정될 때 rendering여부를 결정. return값이 true이면 render()호출. false이면 호출되지 않음
    shouldComponentUpdate(nextProps, nextState){
        // editing값이 변경되지 않고, info값이 변경되지 않으면 render() 호출하지 않음
        if(!this.state.editing && !nextState.editing && nextProps.info===this.props.info){
            return false;
        }
        return true;
    }


    render(){        
        //style 정의
        const style={
            border: "1px solid black",
            padding: '8px',
            margin: '8px'
        };

        // 수정모드인 경우 렌더링
        const { editing }=this.state; // 구조분해할당
        if(editing){
            return(
                <div style={style}>
                    <div>
                        <input
                        value={this.state.name}
                        name="name"
                        placeholder="이름"
                        onChange={this.handleChange}
                        />
                    </div>
                    <div>
                        <input
                        value={this.state.phone}
                        name="phone"
                        placeholder="전화번호"
                        onChange={this.handleChange}
                        />
                    </div>
                    <button onClick={this.handleToggleEdit}>적용</button>
                    <button onClick={this.handleRemove}>삭제</button>
                </div>
            );
        }

        //구조분해할당
        const { name, phone, id }=this.props.info;

        //일반모드인 경우 렌더링       
        return(
            <div style={style}>
                <div><b>{name}</b></div>
                <div>{phone}</div>
                <button onClick={this.handleToggleEdit}>수정</button>
                <button onClick={this.handleRemove}>삭제</button>
                
            </div>
        );
    }
}

export default PhoneInfo;

